import hashlib
import json
import mimetypes
import os
import typing
from dataclasses import dataclass
from pathlib import Path


def cal_sha1_115(f: typing.IO):
    f.seek(0)
    sha1_obj = hashlib.sha1()
    num_bytes = 128 * 1024
    content = f.read(num_bytes)
    if (l := len(content)) < num_bytes:
        content += b'\0' * (num_bytes - l)

    sha1_obj.update(content)
    return sha1_obj.hexdigest().upper()


def cal_md5_baidu(f: typing.IO):
    f.seek(0)
    md5_obj = hashlib.md5()
    num_bytes = 256 * 1024
    content = f.read(num_bytes)
    if (l := len(content)) < num_bytes:
        content += b'\0' * (num_bytes - l)

    md5_obj.update(content)
    return md5_obj.hexdigest().upper()


@dataclass
class MetaNew:
    part: bool = False

    path: Path = None
    raw: typing.IO = None

    # 均为全大写
    size: int = None
    mime: str = None
    md5: str = None
    sha1: str = None
    sha256: str = None

    # 前256K的md5
    md5_baidu: str = None
    # 前128K的sha1
    sha1_115: str = None

    atime: int = None
    mtime: int = None
    ctime: int = None

    @property
    def f(self):
        if self.raw is not None:
            return self.raw

        self.raw = self.path.open('rb')
        stat = self.path.stat()
        self.atime = int(stat.st_atime)
        self.mtime = int(stat.st_mtime)
        self.ctime = int(stat.st_ctime)
        return self.raw

    def cal_part(self) -> bool:
        self.md5_baidu = cal_md5_baidu(self.f)
        self.sha1_115 = cal_sha1_115(self.f)
        self.part = True
        return self.part

    def cal_all(self, buf_size: int = 16384) -> bool:
        if not self.part:
            self.cal_part()

        md5_obj = hashlib.md5()
        sha1_obj = hashlib.sha1()
        sha256_obj = hashlib.sha256()

        self.f.seek(0)
        size = 0
        while True:
            content = self.f.read(buf_size)
            size += len(content)
            if len(content) == 0:
                break

            md5_obj.update(content)
            sha1_obj.update(content)
            sha256_obj.update(content)

        self.size = size
        self.md5 = md5_obj.hexdigest().upper()
        self.sha1 = sha1_obj.hexdigest().upper()
        self.sha256 = sha256_obj.hexdigest().upper()
        return True

    @property
    def mime_type(self) -> str:
        if self.path is not None:
            return mimetypes.guess_type(self.abs_path)[0]

        return 'application/octet-stream'

    @property
    def abs_path(self) -> str:
        return str(self.path.absolute())

    @property
    def basename(self) -> str:
        if self.path is None:
            return 'tmp.txt'

        return self.path.name

    @property
    def suffix(self) -> str:
        return self.path.suffix

    def link_ali(self, name=None) -> str:
        if name is None:
            name = self.basename

        return f"aliyunpan://{name}|{self.sha1}|{self.size}|TMP[Licsber]"

    def link_115(self, name=None) -> str:
        if name is None:
            name = self.basename

        return f"115://{name}|{self.size}|{self.sha1}|{self.sha1_115}"

    def link_baidu(self, name=None) -> str:
        if name is None:
            name = self.basename

        return f"{self.md5}#{self.md5_baidu}#{self.size}#{name}"


class Meta:
    NEED_CHECK = {'size', 'sha1', 'md5', 'sha256', }

    def check(self, other_meta_str: str) -> bool:
        j = json.loads(other_meta_str)
        for check in self.NEED_CHECK:
            if j[check] != self._meta[check]:
                return False

        return True

    def __init__(self, filepath, buf_size=16384):
        abspath = os.path.abspath(filepath)
        if not os.path.exists(abspath):
            print(f"文件不存在 请检查: {abspath}")
            raise FileNotFoundError

        self.buf_size = buf_size

        self._meta = {
            'abspath': abspath,
            'basename': os.path.basename(filepath),
            'size': os.path.getsize(filepath),
            'mime_type': mimetypes.guess_type(abspath)[0],
            'mtime': int(os.path.getmtime(filepath)),
            'md5': '',
            'sha1': '',
            'sha256': '',
            '115_sha1': '',
            'baidu_md5': '',
        }
        self._cal_all()

    @property
    def link_115(self):
        return f"115://{self.basename}|{self.size}|{self.sha1}|{self.head_sha1}"

    @property
    def link_ali(self):
        return f"aliyunpan://{self.basename}|{self.sha1}|{self.size}|TMP[Licsber]"

    @property
    def link_baidu(self):
        return f"{self.md5}#{self.head_md5}#{self.size}#{self.basename}"

    @property
    def link_licsber(self):
        return f"licsber://{self.sha256}-{self.sha1}-{self.size}-{self.basename}"

    @property
    def csv_header(self):
        return 'Key,Filename,Size,SHA1,HeadSHA1,MD5,HeadMD5\n'

    @property
    def csv(self):
        res = self.csv_header
        res += f"Value,{self.basename},{self.size},{self.sha1},{self.head_sha1},{self.md5},{self.head_md5}\n"
        res += f",,,,,,{self.link_115}\n"
        res += f",,,,,,{self.link_ali}\n"
        res += f",,,,,,{self.link_baidu}"
        return res

    def save_meta_csv(self):
        with open(self.abspath + '.licsber.csv', 'w') as f:
            f.write(self.csv)

    def __str__(self):
        res = self._meta.copy()
        del res['abspath']
        res.update({
            'link_115': self.link_115,
            'link_ali': self.link_ali,
            'link_baidu': self.link_baidu,
            'link_licsber': self.link_licsber,
        })
        return json.dumps(res, ensure_ascii=False)

    @property
    def abspath(self):
        return self._meta['abspath']

    @property
    def basename(self):
        return self._meta['basename']

    @property
    def size(self):
        return self._meta['size']

    @property
    def mtime(self):
        return self._meta['mtime']

    @property
    def mime_type(self):
        return self._meta['mime_type']

    @property
    def md5(self):
        return self._meta['md5']

    @property
    def sha1(self):
        return self._meta['sha1']

    @property
    def sha256(self):
        return self._meta['sha256']

    @property
    def head_sha1(self):
        return self._meta['115_sha1']

    @property
    def head_md5(self):
        return self._meta['baidu_md5']

    def _cal_all(self):
        with open(self.abspath, 'rb') as f:
            self._meta['115_sha1'] = cal_sha1_115(f)
            self._meta['baidu_md5'] = cal_md5_baidu(f)

            f.seek(0)
            md5_obj = hashlib.md5()
            sha1_obj = hashlib.sha1()
            sha256_obj = hashlib.sha256()
            while True:
                content = f.read(self.buf_size)
                if not content:
                    break

                md5_obj.update(content)
                sha1_obj.update(content)
                sha256_obj.update(content)

            self._meta['md5'] = md5_obj.hexdigest().upper()
            self._meta['sha1'] = sha1_obj.hexdigest().upper()
            self._meta['sha256'] = sha256_obj.hexdigest().upper()


if __name__ == '__main__':
    test_path = '/tmp/test.licsber'
    with open(test_path, 'w') as f:
        f.write('Hello Licsber.')

    meta = Meta(test_path)
    meta.save_meta_csv()
    print(meta, end='')
    assert meta.link_115 == '115://test.licsber|14|02B02681636CCEDB820385C8A87EA2E1E18ACD5C|C24486ADE0E6AAE9376E4994A7A1267277A13295'



import json
from pathlib import Path


class QuarkAccount:
    save_json_path = Path.home() / '.licsber' / 'quark.json'
    upload_folder_id: str
    cookie: str

    def __init__(self, load: bool = True):
        if load:
            self.load()

    def require_force_login(self):
        print('请登录账号 并输入upload_folder_id和cookie.')
        upload_folder_id = input('upload_folder_id: ').strip(' ')
        if upload_folder_id != '':
            self.upload_folder_id = upload_folder_id

        cookie = input('cookie: ').strip(' ')
        if cookie != '':
            self.cookie = cookie

        self.save()

    def load(self):
        data = json.load(self.save_json_path.open('r'))
        self.upload_folder_id = data['upload_folder_id']
        self.cookie = data['cookie']

    @staticmethod
    def new(root_folder_id: str, cookie: str):
        qa = QuarkAccount(load=False)
        qa.root_folder_id = root_folder_id
        qa.cookie = cookie
        qa.save()
        return qa

    def to_dict(self):
        return {
            'upload_folder_id': self.upload_folder_id,
            'cookie': self.cookie,
        }

    def save(self):
        json.dump(self.to_dict(), self.save_json_path.open('w'))


if __name__ == '__main__':
    qa = QuarkAccount()
    qa.require_force_login()
    print(qa.to_dict())



import hashlib
import json
import os
from pathlib import Path

import tqdm

from licsber.utils.ufile import fun_check_path_exist, save_file
from licsber.utils.umeta import Meta
from licsber.utils.utime import cal_time


@cal_time(output=True)
@fun_check_path_exist(clean=True)
def save_115_dir(start_path=None):
    if os.path.isfile(start_path):
        meta = Meta(start_path)
        print(str(meta))

        save_path = start_path + '.json'
        if not os.path.exists(save_path):
            with open(save_path, 'w') as f:
                f.write(str(meta))
                return

        print('.json文件已存在 开始校验.')
        with open(save_path) as f:
            if meta.check(f.read()):
                print('校验通过.')
            else:
                print('校验失败.')
                with open(start_path + '.new.json', 'w') as f:
                    f.write(str(meta))

        return

    if not os.path.isdir(start_path):
        print(f"选定目录错误: {start_path}")
        return

    def fun_sha1(content):
        sha1_obj = hashlib.sha1()
        sha1_obj.update(content)
        return sha1_obj.hexdigest().upper()

    start_path = Path(start_path)
    save_json_name = 'licsber-bak.json'
    save_path = start_path / save_json_name
    old_path = start_path / 'licsber-bak-old.json'

    exist_sha1 = ''
    if save_path.exists():
        print(f"已存在归档文件: {save_path}")
        exist_sha1 = fun_sha1(save_path.open('rb').read())
        print(f"归档文件sha1: {exist_sha1}")
        os.rename(save_path, old_path)

    def build(root):
        node = {
            'dir_name': os.path.basename(root),
            'dirs': [],
            'files': [],
            'baidu': [],
            'licsber': [],
        }
        all_files = list(os.listdir(root))
        with tqdm.tqdm(total=len(all_files)) as t:
            for i in all_files:
                t.update()
                path = os.path.join(root, i)
                if os.path.isdir(path):
                    # 去除威联通QNAP-NAS中的缓存文件夹
                    if os.path.basename(path) in {'.@__thumb', '@Recycle', '@Transcode'}:
                        continue

                    exists_path = os.path.join(path, save_json_name)
                    if os.path.exists(exists_path):
                        info = json.load(open(exists_path))
                    else:
                        info = build(path)

                    node['dirs'].extend([info])
                elif os.path.isfile(path):
                    if os.path.basename(path) in {save_json_name, 'licsber-bak-old.json'}:
                        continue

                    meta = Meta(path)
                    node['files'].append(meta.link_115.replace('115://', '', 1))
                    node['baidu'].append(meta.link_baidu)
                    node['licsber'].append(meta.link_licsber)

        return node

    res = build(start_path)
    dir_info = json.dumps(res)
    sha1 = fun_sha1(dir_info.encode())
    print(f"归档文件sha1: {sha1}")
    if exist_sha1 and exist_sha1 == sha1:
        print('校验通过.')
        os.remove(old_path)

    save_file(start_path, save_json_name, dir_info)



import base64
from urllib.parse import quote_plus

from pymongo import MongoClient

from licsber.github import get_secret


def get_mongo(pwd_b64=get_secret('L_MONGO_PWD_B64'),
              username=get_secret('L_MONGO_USERNAME'),
              host=get_secret('L_MONGO_HOST'),
              port=get_secret('L_MONGO_PORT'),
              db_name=get_secret('L_MONGO_DEFAULT_DB'),
              connect=True,
              ):
    """
    获取mongo数据库连接, 用于爬虫.
    :param pwd_b64: base64后的密码.
    :param username: 具有数据库权限的用户名.
    :param host: mongodb的url.
    :param port: host的端口.
    :param db_name: 数据库名称.
    :param connect: 是否默认连接.
    :return: mongo数据库连接.
    """
    if not pwd_b64:
        pwd_b64 = input('请输入mongo数据库连接密码(base64表示).')

    pwd = base64.b64decode(pwd_b64).decode('utf-8')
    username = quote_plus(username)
    pwd = quote_plus(pwd)

    conn_str = f"mongodb://{username}:{pwd}@{host}:{port}/{db_name}?retryWrites=true"
    client = MongoClient(
        conn_str,
        uuidRepresentation='standard',
        serverSelectionTimeoutMS=5000,
        connect=connect,
    )

    db = client[db_name]
    return db



import os
from pathlib import Path
from typing import Set, Tuple

import cv2
import numpy as np
import tqdm

from licsber.utils.ucli import check_force
from licsber.utils.ufile import fun_check_path_exist
from licsber.utils.utime import cal_time

BLACKLIST_DIR = {'.@__thumb'}
VIDEO_SUFFIX = {'.mp4', '.mkv', '.avi', '.flv', '.hevc', '.wmv'}
assert '.jpg' not in VIDEO_SUFFIX
COMMON_SUFFIX = {'.jpg', '.nfo', '.ass', '.gif', '.json', '.html', '.iso', '.png'}


def thumb_nail(
        video_path: Path,
        img_num: int = 64,
        map_width: int = 8,
        map_size: Tuple[int, int] = None,
        force: bool = False
) -> bool:
    if not map_size:
        map_size = (2160, 3840, 3)

    save_path = video_path.parent / ('Thumb_' + video_path.name.replace(video_path.suffix, '.jpg'))
    if save_path.exists():
        if save_path.stat().st_size != 8 and not force:
            print(f"\t{save_path} 已存在.")
            return True

    save_path.write_text('Licsber.')
    video = cv2.VideoCapture(str(video_path))
    suc, bgr = video.read()
    if not suc:
        return False

    total_frames = int(video.get(cv2.CAP_PROP_FRAME_COUNT))
    frames_idx = [i for i in range(0, total_frames, total_frames // img_num + 1)]

    height, width, _ = bgr.shape
    vertical = width < height
    if vertical:
        map_width *= 2

    map_height = img_num // map_width
    resize_height = map_size[0] // map_height
    resize_width = map_size[1] // map_width
    resize_shape = (resize_width, resize_height)

    row, col = 0, 0
    res_map = np.zeros(map_size, dtype=np.uint8)
    with tqdm.tqdm(total=len(frames_idx)) as t:
        t.set_description(f"竖版: {vertical} " + video_path.name)
        for idx in frames_idx:
            video.set(cv2.CAP_PROP_POS_FRAMES, idx)
            suc, bgr = video.read()
            if not suc:
                return False

            bgr = cv2.resize(bgr, resize_shape)
            res_map[row * resize_height:(row + 1) * resize_height, col * resize_width:(col + 1) * resize_width] = bgr
            col += 1
            if col >= map_width:
                col = 0
                row += 1

            t.update()

    cv2.imwrite(str(save_path), res_map)
    return True


@cal_time(output=True)
@fun_check_path_exist(clean=True)
def thumb_from_dir(start_path: str = None, video_suffix: Set[str] = None):
    force = check_force()
    if not video_suffix:
        video_suffix = VIDEO_SUFFIX

    if os.path.isfile(start_path):
        thumb_nail(Path(start_path), force=force)
        return

    for root, dirs, files in os.walk(start_path):
        dirs[:] = [d for d in dirs if d not in BLACKLIST_DIR]
        root_path = Path(root)
        files.sort()
        for file in files:
            video_path = root_path / file
            suffix = video_path.suffix.lower()
            if suffix not in video_suffix:
                if suffix not in COMMON_SUFFIX:
                    print(f"{file} 跳过.")

                continue

            suc = thumb_nail(video_path, force=force)
            if not suc:
                print(f"{video_path} 视频错误.")
                if force:
                    continue

                return False



import subprocess
import sys
from pathlib import Path

from licsber.utils.ufile import fun_check_path_exist
from licsber.utils.utime import cal_time

dry_run = False


def merge_videos(video_list_path: Path, save_path: Path):
    video_list_path = str(video_list_path)
    save_path = str(save_path)
    cmd = ['ffmpeg', '-y', '-nostdin',
           '-f', 'concat', '-safe', '0',
           '-i', video_list_path,
           '-c:v', 'copy', '-c:a', 'flac',
           '-strict', '-2',
           save_path]

    if dry_run:
        print(' '.join(cmd))
        return None

    return subprocess.Popen(cmd)


def process_dir(root_path: Path, save_path: Path = None):
    root_path = root_path.resolve(strict=True)
    print(f"开始合并: {root_path}")
    if save_path is None:
        save_path = root_path / (root_path.name + '.mp4')

    files = [f"file '{i.resolve(strict=True)}'" for i in root_path.glob('*.mp4') if i != save_path]
    if len(files) == 0:
        print(f"目录为空: {root_path}")
        return None

    files.sort()
    video_list_path = root_path / 'video_list.txt'
    video_list_path.write_text('\n'.join(files), encoding='utf-8')

    return merge_videos(video_list_path, save_path)


@cal_time(output=True)
@fun_check_path_exist(clean=True)
def merge_dirs(start_path: str = None):
    global dry_run

    dry_run = '-n' in sys.argv

    root = Path(start_path)
    single_merge = len(list(root.glob('*.mp4'))) != 0
    if single_merge:
        print('合并此文件夹的所有视频.')
        sub = process_dir(root, root.parent / (root.name + '.mp4'))
        if sub:
            sub.wait()

        return

    subs = []
    for d in root.iterdir():
        if not d.is_dir():
            continue

        d = d.resolve(strict=True)
        save_path = d.parent / (d.name + '.mp4')
        sub = process_dir(d, save_path)
        if sub:
            subs.append(sub)

    [i.wait() for i in subs]



import functools
import os
import random
import time
from email.utils import formatdate

os.environ['TZ'] = 'Asia/Shanghai'


def gmt_time_http_format() -> str:
    """
    获取当前GMT时间, 格式为[Sun, 06 Nov 2020 08:00:00 GMT].
    :return:
    """
    return formatdate(timeval=None, localtime=False, usegmt=True)


def localtime_date_str() -> str:
    """
    获取当前本地日期, 格式为[2020-12-12].
    :return: 当前本地日期
    """
    now_time = time.localtime()
    return time.strftime('%Y-%m-%d', now_time)


def localtime_str() -> str:
    """
    获取当前本地时间, 格式为[2020-12-12 11:22:33].
    :return: 当前本地时间
    """
    now_time = time.localtime()
    return time.strftime('%Y-%m-%d %H:%M:%S', now_time)


def timestamp_sec() -> int:
    """
    获取1970年到现在UTC的秒数.
    :return: 自1970年1月1日 0点0分0秒以来的秒数.
    """
    now_time = time.time()
    return int(now_time)


def timestamp_mil() -> int:
    """
    获取1970年到现在UTC的毫秒数.
    :return: 自1970年1月1日 0点0分0秒以来的毫秒数.
    """
    now_time = time.time() * 1000 + random.randint(0, 999)
    return int(now_time)


def cal_time(output=False, fps=False):
    """
    计算函数运行时间, 以秒为单位.
    :param output: 是否print输出.
    :param fps: 是否输出fps, 而不是秒数.
    :return: (out, res) 其中res为函数返回值. 当res为None时只返回None.
    """

    def decorator(fun):
        @functools.wraps(fun)
        def wrapper(*args, **kwargs):
            last = time.time()
            res = fun(*args, **kwargs)
            real_time = time.time() - last
            if output:
                if fps:
                    print(f"FPS: {1 / real_time:.3f}")
                else:
                    print(f"执行时间: {real_time:.3f}s")
            if not res:
                return None
            return real_time, res

        return wrapper

    return decorator



import json
import mimetypes

from licsber.drive.quark.account import QuarkAccount
from licsber.spider import get_session
from licsber.utils.utime import gmt_time_http_format, timestamp_mil


class QuarkDriver:
    def update_hash(self, task_id, md5, sha1):
        data = {
            'task_id': task_id,
            'md5': md5.lower(),
            'sha1': sha1.lower(),
        }
        return self.post('/file/update/hash', data=data)

    def upload_auth(self, pre_data, part_number=1):
        task_id = pre_data['task_id']
        data = {
            'auth_info': pre_data['auth_info'],
            'auth_meta': self.auth_meta(
                mine_type=pre_data['callback']['format_type'],
                bucket=pre_data['bucket'],
                obj_key=pre_data['obj_key'],
                part_number=part_number,
                upload_id=pre_data['upload_id'],
            ),
            'task_id': task_id,
        }
        return self.post('/file/upload/auth', data=data)

    def upload_pre(self, pdir_fid, file_name, file_size, mime_type=None):
        if not mime_type:
            mime_type = mimetypes.guess_type(file_name)[0]

        ts_mil = timestamp_mil()
        data = {
            'ccp_hash_update': True,
            'dir_name': '',
            'pdir_fid': pdir_fid,
            'size': file_size,
            'file_name': file_name,
            'format_type': mime_type,
            'l_created_at': ts_mil,
            'l_updated_at': ts_mil,
        }
        return self.post('/file/upload/pre', data=data)

    def __init__(self, account: QuarkAccount):
        self.account = account
        self.base_url = 'https://drive.quark.cn/1/clouddrive'
        self.s = get_session()
        self.params = {
            'pr': 'ucpro',
            'fr': 'pc',
        }
        self.build_session()

    def build_session(self):
        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Referer': 'https://pan.quark.cn/',
            'Content-Type': 'application/json',
            'Cookie': self.account.cookie,
        }
        self.s.headers.update(headers)

    def post(self, path: str, data: dict):
        url = self.base_url + path
        data = json.dumps(data)
        return self.s.post(url, params=self.params, data=data)

    def auth_meta(self, mine_type, bucket, obj_key, part_number, upload_id):
        return f"""PUT

{mine_type}
{gmt_time_http_format()}
x-oss-date:{gmt_time_http_format()}
x-oss-user-agent:aliyun-sdk-js/6.6.1 Chrome 98.0.4758.80 on Windows 10 64-bit
/{bucket}/{obj_key}?partNumber={part_number}&uploadId={upload_id}"""



import os

import torch
import torchvision as tv

from licsber.dl import DATASETS_ROOT

DATASETS_PATH = os.path.join(DATASETS_ROOT, 'pytorch/cifar10')

CLASSES = (
    'plane', 'car', 'bird', 'cat', 'deer',
    'dog', 'frog', 'horse', 'ship', 'truck'
)


def loader(transform_train=None, transform_test=None, batch_size=256, workers=2, pin_memory=True):
    if not transform_train:
        transform_train = tv.transforms.Compose([
            tv.transforms.RandomCrop(32, padding=4),
            tv.transforms.RandomHorizontalFlip(),
            tv.transforms.ToTensor(),
            tv.transforms.Normalize((0.4914, 0.4822, 0.4465), (0.2023, 0.1994, 0.2010)),
        ])

    if not transform_test:
        transform_test = tv.transforms.Compose([
            tv.transforms.ToTensor(),
            tv.transforms.Normalize((0.4914, 0.4822, 0.4465), (0.2023, 0.1994, 0.2010)),
        ])

    # download为True就会打印信息 不想让它打印额外信息
    gz_path = os.path.join(DATASETS_PATH, 'cifar-10-python.tar.gz')
    download = not os.path.exists(gz_path)

    train_datasets = tv.datasets.CIFAR10(DATASETS_PATH, download=download, transform=transform_train)
    test_datasets = tv.datasets.CIFAR10(DATASETS_PATH, train=False, transform=transform_test)

    kwargs = {
        'batch_size': batch_size,
        'num_workers': workers,
        'pin_memory': pin_memory,
    }
    # noinspection PyUnresolvedReferences
    train_loader = torch.utils.data.DataLoader(train_datasets, shuffle=True, **kwargs)
    # noinspection PyUnresolvedReferences
    test_loader = torch.utils.data.DataLoader(test_datasets, shuffle=False, **kwargs)

    return train_loader, test_loader



'''Dual Path Networks in PyTorch.'''
import torch
import torch.nn as nn
import torch.nn.functional as F


class Bottleneck(nn.Module):
    def __init__(self, last_planes, in_planes, out_planes, dense_depth, stride, first_layer):
        super(Bottleneck, self).__init__()
        self.out_planes = out_planes
        self.dense_depth = dense_depth

        self.conv1 = nn.Conv2d(last_planes, in_planes, kernel_size=1, bias=False)
        self.bn1 = nn.BatchNorm2d(in_planes)
        self.conv2 = nn.Conv2d(in_planes, in_planes, kernel_size=3, stride=stride, padding=1, groups=32, bias=False)
        self.bn2 = nn.BatchNorm2d(in_planes)
        self.conv3 = nn.Conv2d(in_planes, out_planes + dense_depth, kernel_size=1, bias=False)
        self.bn3 = nn.BatchNorm2d(out_planes + dense_depth)

        self.shortcut = nn.Sequential()
        if first_layer:
            self.shortcut = nn.Sequential(
                nn.Conv2d(last_planes, out_planes + dense_depth, kernel_size=1, stride=stride, bias=False),
                nn.BatchNorm2d(out_planes + dense_depth)
            )

    def forward(self, x):
        out = F.relu(self.bn1(self.conv1(x)))
        out = F.relu(self.bn2(self.conv2(out)))
        out = self.bn3(self.conv3(out))
        x = self.shortcut(x)
        d = self.out_planes
        out = torch.cat([x[:, :d, :, :] + out[:, :d, :, :], x[:, d:, :, :], out[:, d:, :, :]], 1)
        out = F.relu(out)
        return out


class DPN(nn.Module):
    def __init__(self, cfg):
        super(DPN, self).__init__()
        in_planes, out_planes = cfg['in_planes'], cfg['out_planes']
        num_blocks, dense_depth = cfg['num_blocks'], cfg['dense_depth']

        self.conv1 = nn.Conv2d(3, 64, kernel_size=3, stride=1, padding=1, bias=False)
        self.bn1 = nn.BatchNorm2d(64)
        self.last_planes = 64
        self.layer1 = self._make_layer(in_planes[0], out_planes[0], num_blocks[0], dense_depth[0], stride=1)
        self.layer2 = self._make_layer(in_planes[1], out_planes[1], num_blocks[1], dense_depth[1], stride=2)
        self.layer3 = self._make_layer(in_planes[2], out_planes[2], num_blocks[2], dense_depth[2], stride=2)
        self.layer4 = self._make_layer(in_planes[3], out_planes[3], num_blocks[3], dense_depth[3], stride=2)
        self.linear = nn.Linear(out_planes[3] + (num_blocks[3] + 1) * dense_depth[3], 10)

    def _make_layer(self, in_planes, out_planes, num_blocks, dense_depth, stride):
        strides = [stride] + [1] * (num_blocks - 1)
        layers = []
        for i, stride in enumerate(strides):
            layers.append(Bottleneck(self.last_planes, in_planes, out_planes, dense_depth, stride, i == 0))
            self.last_planes = out_planes + (i + 2) * dense_depth
        return nn.Sequential(*layers)

    def forward(self, x):
        out = F.relu(self.bn1(self.conv1(x)))
        out = self.layer1(out)
        out = self.layer2(out)
        out = self.layer3(out)
        out = self.layer4(out)
        out = F.avg_pool2d(out, 4)
        out = out.view(out.size(0), -1)
        out = self.linear(out)
        return out


def DPN26():
    cfg = {
        'in_planes': (96, 192, 384, 768),
        'out_planes': (256, 512, 1024, 2048),
        'num_blocks': (2, 2, 2, 2),
        'dense_depth': (16, 32, 24, 128)
    }
    return DPN(cfg)


def DPN92():
    cfg = {
        'in_planes': (96, 192, 384, 768),
        'out_planes': (256, 512, 1024, 2048),
        'num_blocks': (3, 4, 20, 3),
        'dense_depth': (16, 32, 24, 128)
    }
    return DPN(cfg)


def test():
    net = DPN92()
    x = torch.randn(1, 3, 32, 32)
    y = net(x)
    print(y)

# test()



import hashlib
import os.path
import re

from bson import ObjectId
from minio import Minio

from licsber.mongo import idd_split_path


class S3Saver:
    def __init__(self, endpoint, access_key, secret_key, bucket_name, mongo_db):
        self.minio = Minio(endpoint, access_key, secret_key)
        if not self.minio.bucket_exists(bucket_name):
            self.minio.make_bucket(bucket_name)

        self.bucket_name = bucket_name
        self.db = mongo_db
        self.db.create_index('suffix')
        self.re = re.compile(r'[a-z0-9]+/[a-z0-9]/[a-z0-9]/([a-z0-9]{24})\..*')

    def fput(self, filepath, save_md5=True, filename=False, dir_name='', custom_dict=None):
        """
        保存文件到minio.
        保存元信息到mongo.
        :param filepath: 文件路径.
        :param save_md5: 是否保存md5.
        :param filename: 是否保存原始文件名.
        :param dir_name: 自定义文件夹分类名.
        :param custom_dict: 自定义字段字典.
        :return: 
        """
        basename = os.path.basename(filepath)
        suffix = os.path.splitext(basename)[-1].replace('.', '', 1)
        doc = {
            'suffix': suffix,
            'size': os.path.getsize(filepath),
        }
        if filename:
            doc['name'] = basename

        if dir_name:
            doc['dir'] = dir_name

        if save_md5:
            doc['md5'] = hashlib.md5(open(filepath, 'rb').read()).hexdigest()

        if custom_dict:
            doc.update(custom_dict)

        doc = self.db.insert_one(doc)
        idd = doc.inserted_id
        dst_path = idd_split_path(idd, suffix)
        self.minio.fput_object(self.bucket_name, dst_path, filepath)

    def del_file(self, idd, suffix):
        self.db.delete_one({'_id': ObjectId(idd)})
        self.minio.remove_object(self.bucket_name, idd_split_path(idd, suffix))

    def dedup(self):
        """
        清除数据库和minio中md5、size都相同的文件.
        :return:
        """
        unique = set()
        for doc in self.db.find(projection=['suffix', 'md5', 'size']).sort([('_id', -1)]):
            if 'md5' not in doc:
                continue

            u = (doc['suffix'], doc['md5'], doc['size'])
            if u not in unique:
                unique.add(u)
            else:
                self.del_file(doc['_id'], doc['suffix'])

    def check_all(self):
        """
        用minio中的所有文件更新数据库.
        :return:
        """
        all_exist_idd = []
        for i in self.minio.list_objects(self.bucket_name, recursive=True):
            path = i.object_name
            if idd := self.re.fullmatch(path):
                idd = ObjectId(idd[1])
                all_exist_idd.append(idd)

        self.db.delete_many({
            '_id': {
                '$nin': all_exist_idd,
            }
        })



import os.path

from licsber.drive.quark import Quark
from licsber.utils.ufile import fun_check_path_exist
from licsber.utils.utime import cal_time


@cal_time(output=True)
@fun_check_path_exist(clean=False)
def upload_by_licsber_json(start_path: str = None):
    if not os.path.isfile(start_path):
        print("[ERROR] The path is not a file.")
        return

    quark = Quark()
    if start_path.endswith('.json'):
        suc = quark.quick_upload_licsber_json(start_path)
    else:
        print(f"[ERROR] The file {start_path} is not a licsber json file.")
        return False

    if suc:
        print("[SUCCESS] Upload success.")
    else:
        print("[ERROR] Upload failed.")



from pathlib import Path

from .bbox import xywh_to_xyxy
from .bbox import xyxy_to_xywh
from .map_into import map_into
from ..github import get_secret

DATASETS_ROOT = get_secret('DATASETS_ROOT', '/datasets/')
CHECKPOINT_ROOT = get_secret('CHECKPOINT_ROOT', '/datasets/checkpoint/')

DATASETS_ROOT = Path(DATASETS_ROOT)
CHECKPOINT_ROOT = Path(CHECKPOINT_ROOT)
DATASETS_ROOT.mkdir(exist_ok=True)
CHECKPOINT_ROOT.mkdir(exist_ok=True)



import json
import mimetypes

from licsber.drive.quark.account import QuarkAccount
from licsber.drive.quark.driver import QuarkDriver


class Quark:
    def __init__(self, account: QuarkAccount = None, debug: bool = False):
        self.debug = debug
        if account is None:
            account = QuarkAccount()

        self.driver = QuarkDriver(account)

    def quick_upload_licsber_json(self, json_path, pdir_fid: str = None) -> bool:
        if pdir_fid is None:
            pdir_fid = self.driver.account.upload_folder_id

        file_meta = json.load(open(json_path, 'r'))
        file_name = file_meta['basename']
        file_size = file_meta['size']
        mime_type = mimetypes.guess_type(file_name)[0]
        if 'mime_type' in file_meta and file_meta['mime_type'] is not None:
            mime_type = file_meta['mime_type']

        if mime_type is None:
            mime_type = 'application/octet-stream'

        res = self.driver.upload_pre(pdir_fid, file_name, file_size, mime_type)
        if res.status_code != 200:
            print('upload pre fail.')
            print(res.headers)
            print(res.text)
            return False

        if self.debug:
            print(f"pre: {res.text}")

        task_id = res.json()['data']['task_id']

        file_md5 = file_meta['md5'].lower()
        file_sha1 = file_meta['sha1'].lower()
        res = self.driver.update_hash(task_id, file_md5, file_sha1)
        if res.status_code != 200:
            print('update hash fail.')
            print(res.headers)
            print(res.text)
            return False

        if self.debug:
            print(f"hash: {res.text}")

        return res.json()['data']['finish']


if __name__ == '__main__':
    test_json_path = 'ila.pdf.json'

    quark = Quark()
    suc = quark.quick_upload_licsber_json(test_json_path)
    print(suc)
